% COCHLEARINPUT2FFTINT calculate intensity spectrum of cochlear input signal
%        
% [STFT_INT] = COCHLEARINPUT2FFTINT(COCHLEAR_INPUT,FS,HOP_TIME_MS)
%   cochlear_input is the signal seen at the cochlea
%   fs is the sampling rate of the input signal
%   hop_time_ms is the time (in ms) between subsequent STFTs (optional
%     argument; default is 1 ms)
%   stft_int is the size [M,N] short-time Fourier transform matrix which
%     has intensity values for each FFT bin and time frame. M is the 
%     number of FFT bins up to fs/2 (M = 1 + FFTlength/2) and N is the 
%     number of time frames
%
% The details of this spectral analysis are done according to:
%     Brian R. Glasberg and Brian C. J. Moore. 
%     "A Model of Loudness Applicable to Time-Varying Sounds" 
%     Journal of the Audio Engineering Society, Vol. 50, No. 5 (May), 2002
%
% In order to have adequate spectral resolution at low frequencies and
% adequate temporal resolution at high frequencies, different sets of FFTs 
% are calculated, using signal segments of different duration (from 2 ms
% to 64 ms) but still aligned at their temporal centers. These sets of 
% STFTs are then concatenated into a single set, according to what segment
% duration are to be used in the FFT analysis in a certain frequency range
% (see loudFFTrangeSpecs.m). For more details, please see pp. 333-334 
% (section 1.2 "Calculation of Running Spectrum and Excitation Pattern").
%
% pph Feb. 17, 2004 (Minor modifications by rjc, Fall 2006)

function [stftInt] = cochlearInput2fftInt(cochleaIn,fs,hopTimeMS)

if (nargin < 3)
  hopTimeMS = 1.0;  % as recommended in the article (p. 334)
end
if (nargin < 2)
  fs = 32e3;
end

loudFFTrangeSpecs;  % read in frequency range/segment duration specifications
windowType = 'hann';  % type of windowing used for signal segments

y = cochleaIn;

fftSize = 2^(nextpow2(fs*max(segDurationsMS)/1000)); % fft length in samples
fftArrayLen = fftSize/2 + 1; % number of FFT bins from 0 to fs/2 Hz

y = y/fftSize; % normalize amplitude by FFT length

% determine number of frequency/temporal resolution regions and the
% length of windowed segments to be used in each fft region 
tempIdx = find(freqRangesHz < fs/2);
nRanges = length(tempIdx); % number of frequency regions
freqRangeEdgesHz = [freqRangesHz(tempIdx) fs/2]; % range edges (Hz)
segDurationsSamp = round(fs*segDurationsMS/1000); % segment durations (samples)
segDurationsSamp = segDurationsSamp(tempIdx);
for i=1:nRanges  % to insure that all segment lengths are even
  if (mod(segDurationsSamp(i),2) ~= 0)
    segDurationsSamp(i) = segDurationsSamp(i)+1; % add 1 more sample
  end
end

maxSegLen = max(segDurationsSamp); % longest segment length used in an fft
segOffset = maxSegLen/2; % offset of signal vector in samples 
nFrames = ceil(length(y)/fs*(hopTimeMS*1000)) + 1; % number of time frames
centerTimeMS = (0:nFrames-1)*hopTimeMS; % time centers of fft segments 
centerSample = round(fs*centerTimeMS/1000) + segOffset; % 2nd half of segment
                                                        % begins on this sample

% adjusting signal buffer to allow the first time frame to be centered
% at 0 seconds
yTemp = zeros(1,maxSegLen + ceil(nFrames*fs*hopTimeMS/1000));
yTemp((segOffset+1):(segOffset + length(y))) = y;

% calculate different-resolution FFTs for each time frame
fftMatrix = zeros(fftArrayLen,nFrames,nRanges);
for i=1:nRanges
  eval(['segWindow = ' windowType '(segDurationsSamp(i));']);
  segBuffer = zeros(1,fftSize);  
  tempOffset = segDurationsSamp(i)/2;
  for j=1:nFrames
    segment = yTemp(centerSample(j)-tempOffset+1:centerSample(j)+tempOffset);
    segBuffer(1:segDurationsSamp(i)) = segWindow'.*segment;
    fftBuffer= fft(segBuffer,fftSize);
    fftMatrix(:,j,i) = fftBuffer(1:fftArrayLen);
  end
end
fftMatrixInt = fftMatrix(:,:,:).*conj(fftMatrix(:,:,:));

% construct new FFT matrix whose intensity values in different frequency 
% regions correspond to the intensity values calculated from the different 
% FFTs in fftMatrixInt
freqRangeArray = zeros(1,fftArrayLen); % which freq range each bin belongs to
freqArrayHz = fs * (0:(fftSize/2))/fftSize; % frequency (Hz) of each bin
fftMatrixIntComb = zeros(fftArrayLen,nFrames);
for i=1:fftArrayLen
  freqRangeArray(i) = max(find(freqArrayHz(i) >= freqRangeEdgesHz(1:nRanges)));
end
for i=1:nRanges
  idx = find(freqRangeArray == i);
  fftMatrixIntComb(idx,:) = fftMatrixInt(idx,:,i);
end

stftInt = fftMatrixIntComb;
