% EXCITPAT2SPECLOUD calculate specific loudness from excitation pattern
% [SPEC_LOUD_INT] = EXCITPAT2SPECLOUD(EXCIT_PAT_INT,CENTER_ARRAY_HZ)
%   excit_pat_int is the size [M,N] excitation pattern matrix (in units
%     of intensity or power), where M is the number of ERB bands and N
%     is the number of time frames
%   center_array_hz is vector of center frequencies (in Hz) corresponding
%     to each ERB band
%   spec_loud_int is the size [M,N] specific loudness matrix (in units
%     of intensity or power) corresponding to the input excitation pattern
%
% Specific loudness is described as the loudness per ERB. This function
% assumes we're calculating for sounds in quiet rather than sounds in noise.
% The transformation from excitation pattern to specific loudness 
% involves a compressive nonlinearity, meant to resemble the compression
% which occurs in the cochlea.
% 
% Equations and parameter values taken from pp. 227-231 (sections 1.6
% "Transformation from Excitation to Specific Loudness for Sounds in 
% Quiet" and 1.10 "Values of the Constants") of:
%     Brian C. J. Moore, Brian R. Glasberg, and Thomas Baer. 
%     "A Model for the Prediction of Thresholds, Loudness, and Partial
%        Loudness" 
%     Journal of the Audio Engineering Society, Vol. 45, No. 4 (April), 1997
%
% pph Feb. 3, 2004

function [specLoudInt] = excitPat2SpecLoud(excitPatInt,bandCentersHz)

E_thrq_500Hz = 2.31; % for monaural listening (about 3.6 dB)
                     % (p. 228, 1st paragraph) 
C = 0.047; % chosen so that loudness of a 1 kHz sinusoid presented
           % binaurally in free feield at 40 dB SPL is exactly 1 sone
           % (p. 231, section 1.10)

[nBands,nFrames] = size(excitPatInt);
specLoudInt = zeros(nBands,nFrames);

for i=1:nBands
  freqHz = bandCentersHz(i);
  for k=1:nFrames
    E_sig = excitPatInt(i,k); % in power or intensity units

    if (freqHz >= 500)  % for frequencies above or equal to 500 Hz
      E_thrq = E_thrq_500Hz;
      G = 1; % in amplitude units
      A = 2*E_thrq;
      alpha = 0.2;
    else  % for frequencies below 500 Hz
      E_thrq = getLoudParamValues(freqHz,'Ethrq'); % see Figs. 4 & 8
      G = E_thrq_500Hz/E_thrq; % (p. 227, last paragraph)
      if (G > 1.0) G = 1.0; end % in case G > 0 dB
      A = getLoudParamValues(G,'A'); % depends on G; see Fig. 7
      alpha = getLoudParamValues(G,'alpha'); % depends on G; see Fig. 6
    end

    % three different cases: when E_sig is very soft, in a moderate range,
    % or very large
    if (E_sig < E_thrq)  % eq. 8
      N = C * ((G*E_sig + A)^alpha - A^alpha) * (2*E_sig/(E_sig+E_thrq))^1.5;
    elseif (E_sig > 10^10) % eq. 9
      N = C * sqrt(E_sig*1.04*10^-6);
    else  % eq. 6
      N = C * ((G*E_sig + A)^alpha - A^alpha);
    end

    specLoudInt(i,k) = N;    

  end
end