% FFTINT2BANDDB calculate total intensity in ERB bands from FFT
% [BAND_DB, BAND_CENTERS_HZ] = FFTINT2BANDDB(STFT_INT,FS,ERB_INC,START_ERB)
%   stft_int is the size [M,N] short-time Fourier transform matrix which
%     has intensity values for each FFT bin and time frame. M is the 
%     number of FFT bins up to fs/2 (M = 1 + FFTlength/2) and N is the 
%     number of time frames. (usually generated by cochlearInput2fftInt.m)
%   fs is the sampling rate of the input signal
%   erb_inc is the spacing interval (in ERBs) between the ERB band centers.
%     A smaller value (e.g., 0.1) will have more overlap. In the 1990 
%     article, erb_inc = 0.1, while in the 2002 article, erb_inc = 0.25.
%     This is an optional argument; the default is set to 0.25.
%   start_erb is the center frequency (in ERBs) of the lowest-frequency
%     ERB band of interest. This optional argument has a default of
%     1.0 ERB, or about 26 Hz.
%   band_db is the size [L,N] matrix which has values of the total 
%     intensity in each ERB band (converted to dB) and time frame. L
%     is the number of ERB bands and N is the number of time frames.
%   band_centers_hz is the array of center frequencies (in Hz) of all the
%     ERB bands
% 
% The total intensity in each ERB band is calculated, by summing the 
% intensities of all components which fall within +/- 0.5 ERB of each 
% band center. The bands cover the frequency range from start_ERB to
% half the sampling rate. The total intensity is then converted to dB,
% with the lowest decibel level kept at -100 dB.
%
% Main resource: p. 136 (Fortran code) of
%     Brian R. Glasberg and Brian C. J. Moore. 
%     "Derivation of auditory filter shapes from notched-noise data"
%     Hearing Research, Vol. 47 pp. 103-138, 1990 
%
% pph Feb. 3, 2004

function [bandDB,bandCentersHz] = fftInt2BandDB(stftInt,fs,ERBinc,startERB)

if (nargin < 3)
  ERBinc = 0.25;
end
if (nargin < 4)
  startERB = 1.0;  % about 26 Hz
end

minIntensity = 10^-10;
deviationERB = 0.5;

[fftArrayLen,nFrames] = size(stftInt);
fftSize = 2*(fftArrayLen-1);
freqArrayHz = fs * (0:(fftSize/2))/fftSize;
freqArrayERB = freq2erb(freqArrayHz);

endERB = floor((freq2erb(fs/2)-deviationERB)/ERBinc)*ERBinc;
bandCentersERB = startERB:ERBinc:endERB; % array of band centers in ERB
bandCentersHz = erb2freq(bandCentersERB);
nBands = length(bandCentersERB);

bandDB = zeros(nBands,nFrames);

for i=1:nBands
  centerERB = bandCentersERB(i);
  tempIdx1 = find(freqArrayERB > (centerERB-deviationERB));
  tempIdx2 = find(freqArrayERB < (centerERB+deviationERB));
  tempIdx = intersect(tempIdx1,tempIdx2);
  intensityArray = sum(stftInt(tempIdx,:),1); % total intensity in a band
  idx = find(intensityArray < minIntensity);
  intensityArray(idx) = minIntensity;
  bandDB(i,:) = 10*log10(intensityArray);
end
