% FFTINT2EXCITPAT calculate excitation pattern from FFT
% [EXCIT_PAT_INT,BAND_CENTERS_HZ] = FFTINT2EXCITPAT(STFT_INT,FS,ERB_INC,PRINT_PROGRESS_FLAG)
%   stft_int is the size [M,N] short-time Fourier transform matrix which
%     has intensity values for each FFT bin and time frame. M is the 
%     number of FFT bins up to fs/2 (M = 1 + FFTlength/2) and N is the 
%     number of time frames. (usually generated by cochlearInput2fftInt.m)
%   fs is the sampling rate of the input signal
%   erb_inc is the spacing interval (in ERBs) between the ERB band centers.
%     A smaller value (e.g., 0.1) will have more overlap. In the 1990 
%     article, erb_inc = 0.1, while in the 2002 article, erb_inc = 0.25.
%     This is an optional argument; the default is set to 0.25.
%   print_progress_flag is an optional argument, to print out the 
%     progress of the computation, since it may be kind of slow. 0 for no
%     printouts (the default mode), 1 for printing into the matlab terminal.
%   excit_pat_int is the size [M,N] excitation pattern matrix (in units
%     of intensity or power), where M is the number of ERB bands and N
%     is the number of time frames
%   band_centers_hz is the array of center frequencies (in Hz) corresponding
%     to each ERB band
%
% Main resource: p. 114 ("The variation of the auditory filter shape 
% with level" section)and p. 137 (Fortran code) of
%     Brian R. Glasberg and Brian C. J. Moore. 
%     "Derivation of auditory filter shapes from notched-noise data"
%     Hearing Research, Vol. 47 pp. 103-138, 1990 
% Other resource:
%     Brian C. J. Moore, Brian R. Glasberg, and Thomas Baer. 
%     "A Model for the Prediction of Thresholds, Loudness, and Partial
%        Loudness" 
%     Journal of the Audio Engineering Society, Vol. 45, No. 4 (April), 1997
% 
% The excitation pattern of the signal is calculated from its FFT analysis.
% Auditory filter shapes are a function of center frequency and the 
% level of the components exciting the filters. The high-frequency skirt
% of the auditory filter are assumed to be consistent with level, while
% the low-frequency skirt becomes less sharp with increasing level. (This
% is represented by the parameter p in the code below.) Frequency components 
% far above a certain ERB band (when g > 2.0) are considered to contribute 
% nothing to the excitation level at that band. To keep the values within 
% a reasonable range, any intensity level lower than 10^-10 (-100 dB) is 
% replaced with that value.
%
% pph Feb. 3, 2004 (Minor modifications by rjc, Fall 2006)

function [excitPatInt,bandCentersHz] = fftInt2ExcitPat(stftInt,fs,ERBinc,printProgressFlag)

if (nargin < 2)
  fs = 32e3;
end
if (nargin < 3)
  ERBinc = 0.25;
end
if (nargin < 4)
  printProgressFlag = 0;
end

p_51at1k = 4*1000/(24.7 * (4.37 + 1)); % 4*f_c/erbWidth at f_c = 1000 Hz
minIntensity = 10^-10;

% calculate total intensity in each ERB band
[bandDB,bandCentersHz] = fftInt2BandDB(stftInt,fs,ERBinc);

[nBands,nFrames] = size(bandDB);
[fftArrayLen,nFrames] = size(stftInt);
fftSize = 2*(fftArrayLen-1);
bandCentersERB = freq2erb(bandCentersHz);
freqArrayHz = fs * (0:(fftSize/2))/fftSize;
freqArrayERB = freq2erb(freqArrayHz);

% find the index of the array of band centers closest to each FFT bin freq 
closestERBidx = zeros(1,fftArrayLen);
for i=1:fftArrayLen
  [temp,closestERBidx(i)] = min(abs(bandCentersERB-freqArrayERB(i)));
end

excitPatInt = zeros(nBands,nFrames);

for i=1:nBands

  if (printProgressFlag == 1)
    disp(['ERB band ', int2str(i), ' out of ', int2str(nBands), ' bands.']);
  end

  f_c = bandCentersHz(i); % center frequency of band
  erbWidth = freq2erbWidthHz(f_c);
  p_51 = 4*f_c/erbWidth;
  p_scalar = 0.35*(p_51/p_51at1k); % 1997 article uses 0.35, not 0.38 (p. 226)
  g = (freqArrayHz - f_c)/f_c; % normalized deviation from band center
  tempIntensity = zeros(1,nBands);
  p = p_51*ones(1,fftArrayLen);
  pl_idx = find( g < 0.0); % index array for lower skirt of auditory filter

  for k=1:nFrames    
    p(pl_idx) = p_51 - p_scalar.*(bandDB(closestERBidx(pl_idx),k) - 51);
    idx = find(g < 2.0);
    g = abs(g);
    pg = p(idx).*g(idx);
    tempIntensity = (1.0 + pg).*exp(-pg).*stftInt(idx,k)';
    excitPatInt(i,k) = sum(tempIntensity);
  end

end

% limit the lowest intensity value to 10^-10
[I,J] = find(excitPatInt < minIntensity);
if (I)
  excitPatInt(I,J) = minIntensity;
end

