% GETSPECLOUDPARAMVALUES return specific loudness parameter values
% [OUT_VAL] = GETSPECLOUDPARAMVALUES(IN_VAL,DESIRED_PARAM_TYPE)
%   This is a catch-all function to return values needed in the
%   calculation of specific loudness in ERB bandcenters which are below
%   500 Hz. The parameters are "A", "alpha", and "E_thrq" (see function
%   excitPat2SpecLoud.m). The values do not have a closed-form solution;
%   rather, they are calculated by interpolation of values taken from plots
%   in the 1997 AES Moore, Glasberg, and Baer paper (see function
%   moore_AES97_graphValues.m).
%   
%   desiredParamType: a string to indicate what parameter is desired
%     - 'A' for parameter A
%     - 'alpha' for parameter alpha
%     - 'Ethrq' for parameter E_thrq
%   inVal and outVal: input and output values 
%     - input G (amplitude), output A or alpha
%     - input frequency (Hz), output E_thrq (intensity/power)
% 
% pph Feb. 3, 2004

function [outVal] = getSpecLoudParamValues(inVal,desiredParamType)


moore_AES97_graphValues; % load in values
errorFlag = 0;

switch desiredParamType
  case 'A'
    xVal = 10*log10(inVal);
    xVector = GPointsDB;
    yVector = APoints;
  case 'alpha'
    xVal = 10*log10(inVal);
    xVector = GPointsDB;
    yVector = alphaPoints;
  case 'Ethrq'
    xVal = inVal;
    xVector = E_thrqHz;
    yVector = E_thrqDB;
  otherwise
    errorFlag = 1;    
end

% if no interpolation is needed...
idx = find(xVector == xVal);
if ( length(idx) == 1)
  outVal = yVector(idx);
  return
end

% if interpolation is needed...
tempIdx = find(xVector > xVal);
idx = tempIdx(1);
if (idx == 1)  % smaller x values off the scale are interpolated from 
  idx = 2;     % the lowest two x points
end
idxPair = [idx-1,idx];
if (idxPair(2) > length(xVector))  % cannot calculate for x values
  errorFlag = 1;                   % greater than the range in xVector
end

if (errorFlag == 1)
  disp('The function getSpecLoudParamValues was not able to calculate your desired parameter value');
  return
end


if (desiredParamType == 'Ethrq')
  xVal1 = xVector(idxPair(1));
  xVal2 = xVector(idxPair(2));
  scaleFactor = (log10(xVal)-log10(xVal1))/(log10(xVal2)-log10(xVal1));
  xValCorr = scaleFactor*(xVal2-xVal1) + xVal1; % to account for log axis
  tempVal = genericInterp(yVector(idxPair),xVector(idxPair),xValCorr); % in dB
  outVal = 10^(tempVal/10); % convert to linear intensity
else
  outVal = genericInterp(yVector(idxPair),xVector(idxPair),xVal);
end
