% LOUDLONGTERM  calculate long-term loudness from short-term loudness
% [LOUDNESS_LT] = LOUDLONGTERM(LOUDNESS_ST,FRAME_HOP_MS,DURATION_S,ATTACK_TIME_MS,RELEASE_TIME_MS)
%   loudness_ST is the short-term loudness vector calculated with the 
%     function loudShortTerm.m
%   frame_hop_ms is the time interval between each sample (frame) of the 
%     vector loudness_ST, in milliseconds
%   duration_s indicates how long (in seconds) the long-term loudness should 
%     be computed; default is the same duration as loudness_ST
%   attack_time_ms is the attack time of integration used for calculating
%     long-term loudness, in milliseconds; default is about 99.5 ms
%   release_time_ms is the release time (in ms) used for calculating long-term
%     loudness; default is about 2 s
%   loudness_L is the estimated long-term loudness of a signal which has 
%     the short-term loudness profile loudness_ST; has units in sones
%
% pph Feb. 3, 2004

function [loudnessLT] = loudLongTerm(loudnessST,frameHopMS,durationS,attackTimeMS,releaseTimeMS)

	
T_i = frameHopMS; 

% number of loudness frames to compute
if (nargin < 3)
  nFrames = length(loudnessST);  
else
  nFrames = floor(frameHopMS*durationS*1000);  % frames within desired duration
end

% Default values taken from p. 335 of: 
%     Brian R. Glasberg and Brian C. J. Moore. 
%     "A Model of Loudness Applicable to Time-Varying Sounds" 
%     Journal of the Audio Engineering Society, Vol. 50, No. 5 (May), 2002
% T_i = 1 ms; alpha_a = 0.01; alpha_r = 0.0005
if (nargin < 4)
  T_a = -1.0/(log(1-0.01));
end
if (nargin < 5)
  T_r = -1.0/(log(1-0.0005));
end

% calculate constants used in calculating long-term loudness
alpha_a = 1 - exp(-T_i/T_a);  % constant related to attack time T_a
alpha_r = 1 - exp(-T_i/T_r);  % constant related to release time T_r
beta_a = 1-alpha_a; 
beta_r = 1-alpha_r;

S_p = zeros(1,nFrames+1);  % short-term loudness vector
S_pp = zeros(1,nFrames);  % long-term loudness vector

tempN = min(nFrames,length(loudnessST));
S_p(1:tempN) = loudnessST(1:tempN);

% calculating long-term loudness
for n=1:nFrames
  if (S_p(n+1) > S_pp(n))
    S_pp(n+1) = alpha_a*S_p(n+1) + beta_a*S_pp(n); % "attack" phase 
  else 
    S_pp(n+1) = alpha_r*S_p(n+1) + beta_r*S_pp(n); % "release" phase
  end
end

loudnessLT = S_pp;
