% LOUDSHORTTERM  calculate short-term loudness from "instantaneous" loudness
% [LOUDNESS_ST] = LOUDSHORTTERM(LOUDNESS_I,FRAME_HOP_MS,DURATION_S,ATTACK_TIME_MS,RELEASE_TIME_MS)
%   loudness_I is the vector resulting from the integration of the specific 
%     loudness pattern calculated with the function loudInstant.m
%   frame_hop_ms is the time interval between each sample (frame) of the 
%     vector loudness_I, in milliseconds
%   duration_s indicates how long (in seconds) the short-term loudness should 
%     be computed; default is the same duration as loudness_I
%   attack_time_ms is the attack time of integration used for calculating
%     short-term loudness, in milliseconds; default is about 21.7 ms
%   release_time_ms is the release time (in ms) used for calculating short-term
%     loudness; default is about 49.5 ms
%   loudness_ST is the estimated short-term loudness of a signal which has 
%     the instantaneous loudness profile loudness_I; has units in sones
%
% pph Feb. 3, 2004

function [loudnessST] = loudShortTerm(loudnessI,frameHopMS,durationS,attackTimeMS,releaseTimeMS)

	
T_i = frameHopMS; 

% number of loudness frames to compute
if (nargin < 3)
  nFrames = length(loudnessI);  
else
  nFrames = floor(frameHopMS*durationS*1000);  % frames within desired duration
end

% Default values taken from p. 334 of: 
%     Brian R. Glasberg and Brian C. J. Moore. 
%     "A Model of Loudness Applicable to Time-Varying Sounds" 
%     Journal of the Audio Engineering Society, Vol. 50, No. 5 (May), 2002
% T_i = 1 ms; alpha_a = 0.045; alpha_r = 0.02
if (nargin < 4)
  T_a = -1.0/(log(1-0.045));
end
if (nargin < 5)
  T_r = -1.0/(log(1-0.02));
end

% calculate constants used in calculating short-term loudness
alpha_a = 1 - exp(-T_i/T_a);  % constant related to attack time T_a
alpha_r = 1 - exp(-T_i/T_r);  % constant related to release time T_r
beta_a = 1-alpha_a; 
beta_r = 1-alpha_r;

S = zeros(1,nFrames+1);  % instantaneous loudness vector
S_p = zeros(1,nFrames);  % short-term loudness vector

tempN = min(nFrames,length(loudnessI));
S(1:tempN) = loudnessI(1:tempN);

% calculating short-term loudness
for n=1:nFrames
  if (S(n+1) > S_p(n))
    S_p(n+1) = alpha_a*S(n+1) + beta_a*S_p(n); % "attack" phase 
  else 
    S_p(n+1) = alpha_r*S(n+1) + beta_r*S_p(n); % "release" phase
  end
end

loudnessST = S_p;