%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Institution: Stanford University
% Project: RealSimple Psychoacoustics Lab
% Author: Ryan J. Cassidy
% Date: Fall 2006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SPECLOUD2LOUDPROF Converts an input specific loudness-gram to an
% instantaneous, short-term, and long-term loudness profile, and plots the
% results.
%
% specLoud2LoudProf
%
%    - specLoudInt is the specific loudness-gram, and
%    - BandCentersHz are the frequencies at which the specLoudInt is computed.
%
% [1] B. R. Glasberg and B. C. J. Moore, "A model of loudness applicable to
% time-varying sounds", Journal of the Audio Engineering Society,
% 50(5):331--342, 2002.
% 
% VERSION CONTROL INFORMATION:
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function specLoud2LoudProf(specLoudInt, bandCentersHz)

ERBinc = 0.25; % increment in ERBs
hopTimeMS = 1; % time between frames, in milliseconds
durScaler = 2.0; % how much longer/shorter of a time period the
                 % loudness should be computed over
sigDuration = 0.4; % duration of signal in seconds (TODO_RJC: Fix this
                   % hard-coding). 
loudDuration = durScaler*sigDuration;

instantLoudSone = loudInstant(specLoudInt,ERBinc);
%instantLoudSone = 2*instantLoudSone; % uncomment to "fake" a binaural signal
shortLoudSone = loudShortTerm(instantLoudSone,hopTimeMS,loudDuration);
longLoudSone = loudLongTerm(shortLoudSone,hopTimeMS,loudDuration);

% for plotting the loudness
maxLen = length(longLoudSone);
loudMatrixSone = zeros(3,maxLen);
loudMatrixSone(1,1:length(instantLoudSone)) = instantLoudSone;
loudMatrixSone(2,1:length(shortLoudSone)) = shortLoudSone;
loudMatrixSone(3,1:length(longLoudSone)) = longLoudSone;
loudMatrixPhon = 40 + 10*log2(loudMatrixSone+eps);
timeAxis = hopTimeMS*(0:maxLen-1); % in ms

% Plot loudness profile.
fig_num_base = 10;
figure(fig_num_base+1);clf;
printplots = 0;      % 1 to print, 0 to skip
subplot(2,1,1);
plot(timeAxis,loudMatrixSone(1,:),'k');hold on;
plot(timeAxis,loudMatrixSone(2,:),'r--');
plot(timeAxis,loudMatrixSone(3,:),'b-.');grid;zoom on;
title('Loudness on Different Time Scales');
xlabel('Time (ms)');ylabel('Loudness (sones)');
legend('instantaneous','short-term','long-term');
subplot(2,1,2);
if printplots, 
  plot(timeAxis,loudMatrixPhon(1,:),'k');hold on;
  plot(timeAxis,loudMatrixPhon(2,:),'--k');
  plot(timeAxis,loudMatrixPhon(3,:),'-.k');grid;zoom on;
else
  plot(timeAxis,loudMatrixPhon(1,:),'k');hold on;
  plot(timeAxis,loudMatrixPhon(2,:),'r--');
  plot(timeAxis,loudMatrixPhon(3,:),'b-.');grid;zoom on;
end
xlabel('Time (ms)');ylabel('Loudness level (phons)');
legend('instantaneous','short-term','long-term');
if printplots, 
  cmd = sprintf('print -deps %s/lsg-isl.eps',pdir); 
  disp(cmd); eval(cmd);
end
ylim([-10,50]);
